''''''''''''''''''''''''''''''''''''''''''''
' Borg.vbs
' Version 1.1 of 2017-02-06
'
' Tool to document Active Directory sites
' in an HTML report
'
' by Nils Kaczenski (firstname at lastname .de)
' http://www.faq-o-matic.net
'
' No warranty! Use at your own risk!
' Keine Gewhr! Nutzung auf eigene Gefahr!
'
' To Do List:
'
' Changes:
'			- 1.1 corrects Error In GC detection (previously, a DC could be identified as GC even if it wasn't)
'			      adds newer AD opmodes
'			- 1.0 Re-versioned for publication
'			- 0.77 Fixed problem with weird non-empty GPO links that do not link to GPOs ...
'			- 0.76 Fixed incorrect subnet count, added some more counters
'			- 0.75 Bugfix: Sites without Site Links used to cause an exception
'			- 0.74 Added statistics
'			- 0.74 Added Page Size for ADO queries
' 			- 0.73 Added "Longhorn" Functional Modes
'			- 0.72 Completed forest and FSMO information
'			- 0.72 added Server OS lookup
'			- 0.72 fixed bug in Bridgehead Server determination
'			- 0.71 new option /showReport opens report on completion
'			- 0.71 display Created and Modified on each relevant object
'			- 0.71 simplified GC check, removed /showGC option
'			- 0.71 checks for presence of NTDS Settings to tell DCs from other servers
'			- 0.71 checks for "description" array
'			- added process information for GC retrieval
'			- added /showGC option
'
'''''''''''''''''''''''''''''''''''''''''''''

' Globale Variablen und Objekte
Const NTDSDSA_OPT_IS_GC = 1
Dim objDatei
Dim pfadName
Dim intSiteZahl
Dim intServerZahl
Dim intSiteLinkZahl
Dim intSubnetZahl

' HTML-Symbolpfade
strSiteSym    		= "<img src=""../Dateien/site.gif"">&nbsp;"
strnTDSSiteSettingsSym    		= "<img src=""../Dateien/nTDSSiteSettings.gif"">&nbsp;"
strSiteLinkSym    	= "<img src=""../Dateien/siteLink.gif"">&nbsp;"
strSubnetSym    	= "<img src=""../Dateien/subnet.gif"">&nbsp;"
strDCSym       	= "<img src=""../Dateien/server.gif"">&nbsp;"
strGPOSym           = "<img src=""../Dateien/gpo.gif"">&nbsp;"

' Einrckungseinheit
intIndentPx = 25

' Skriptname
strTitel = "Borg 1.1"

' have we been called by cscript?
If InStr(1, WScript.FullName, "cscript", vbTextCompare) = 0 Then 
	' no - restart correctly
	Set objShell = CreateObject("WScript.Shell")
	WScript.Quit objShell.Run( "cscript " & """" & WScript.ScriptFullName, , True)
End If

' fetch CMD arguments
For Each strArg In WScript.Arguments
	strArg = LCase(strArg)
	strArg = Replace(strArg, "/", "")
	strArg = Replace(strArg, "-", "")
	Select Case strArg
		Case "showreport" blnShowReport = True
	End Select
Next

' check if ADSI error messages are available (ADSI Resource Kit of 1998)
' see: http://www.it-administrator.de/artikel/17389.html
' download from: http://www.microsoft.com/technet/archive/winntas/downloads/adsi25.mspx?mfr=true
On Error Resume Next
Set ADError = Nothing
Set ADError = CreateObject("ADsError")
On Error Goto 0
If ADError Is Nothing Then
   ADSIError = False
Else
   ADSIError = True
End If
Set ADError = Nothing


' Connect to forest
Set objRoot = GetObject("LDAP://rootDSE")
strConfig = objRoot.Get("configurationNamingContext")
strRootDom = objRoot.Get("rootDomainNamingContext")
strDomain = objRoot.Get("defaultNamingContext")
strSchema = objRoot.Get("schemaNamingContext")
strSites = "CN=Sites," & strConfig
Set objSites = GetObject("LDAP://" & strSites)


' Hauptprozedur
AusgabeDatei          ' Ausgabedatei ffnen
listRoot
listSites  ' Daten listen
listSymbols
dateiZu               ' Datei schlieen
say "Finished at " & Now & "."      ' Erfolgsmeldung

Sub listRoot
	strForLevel = ""
	strDomLevel = ""
	strDomMixed = ""
	strSchemaVersion = ""

    On Error Resume Next
	   	' Forest-Mode gibt es erst ab 2003, daher Fehlerprfung
		strDomLevel = objRoot.Get("domainFunctionality")
  	  	strForLevel = objRoot.Get("forestFunctionality")
      	Set objRootDom = GetObject("LDAP://" & strRootDom)
		dtmRootCreateTimestamp = objRootDom.Get("whenCreated")
		dtmRootModifyTimestamp = objRootDom.Get("whenChanged")
	On Error Goto 0

	Select Case strForLevel
		Case "0" strForLevel = "Windows 2000 Mixed (DCs: NT, 2000, 2003)"
		Case "1" strForLevel = "Windows Server 2003 Interim (DCs: NT, 2003)"
		Case "2" strForLevel = "Windows Server 2003 Native (DCs: 2003 only)"
		Case "3" strForLevel = "Windows Server 2008 (DCs: 2008 or later)"
		Case "4" strForLevel = "Windows Server 2008 R2 (DCs: 2008 R2 or later)"
		Case "5" strForLevel = "Windows Server 2012 (DCs: 2012 or later)"
		Case "6" strForLevel = "Windows Server 2012 R2 (DCs: 2012 R2 or later)"
		Case "7" strForLevel = "Windows Server 2016 (DCs: 2016 or later)"
		Case Else strForLevel = "Forest Mode " & strForLevel
	End Select

	Select Case strDomLevel
		Case "0" strDomLevel = "Windows 2000"
		Case "1" strDomLevel = "Windows Server 2003"
		Case "2" strDomLevel = "Windows Server 2003"
		Case "3" strDomLevel = "Windows Server 2008"
		Case "4" strDomLevel = "Windows Server 2008 R2"
		Case "5" strDomLevel = "Windows Server 2012"
		Case "6" strDomLevel = "Windows Server 2012 R2"
		Case "7" strDomLevel = "Windows Server 2016"
		Case Else strDomLevel = "Domain Mode " & strDomLevel
	End Select

	Set objSchema = GetObject ("LDAP://" & strSchema)
	strSchemaMaster = GetValue(GetValue(objSchema.Get("fSMORoleOwner"), "serverReferenceBL"), "name")
	
	Set objPartitions = GetObject("LDAP://CN=Partitions," & objRoot.Get("configurationNamingContext"))
	strDomainNamingMaster = GetValue(GetValue(objPartitions.Get("fSMORoleOwner"), "serverReferenceBL"), "name")
	
	Set objDomain = GetObject("LDAP://" & strDomain)
	strPdcEmulator = GetValue(GetValue(objDomain.Get("fSMORoleOwner"), "serverReferenceBL"), "name")
	dtmDomCreateTimestamp = objDomain.Get("whenCreated")
	dtmDomModifyTimestamp = objDomain.Get("whenChanged")
	
	Set objRidManager = GetObject("LDAP://CN=RID Manager$,CN=System," & strDomain)
	strRidMaster = GetValue(GetValue(objRidManager.Get("fSMORoleOwner"), "serverReferenceBL"), "name")

	Set objInfrastructure = GetObject("LDAP://CN=Infrastructure," & strDomain)
	strInfrastructureMaster = GetValue(GetValue(objInfrastructure.Get("fSMORoleOwner"), "serverReferenceBL"), "name")

	SchreibDatei "<div>Forest Functional Mode: <I>" & strForLevel& "</I></div>"
	SchreibDatei "<div>Root Domain: <I>" & strRootDom & "</I></div>"
	SchreibDatei "<div>Root Domain Created: <I>" & dtmRootCreateTimestamp & "</I>, Modified: <I>" & dtmRootModifyTimestamp & "</I></div>"
	SchreibDatei "<div>Root Domain Functional Mode: <I>" & strDomLevel& "</I></div>"
	SchreibDatei "<div>Current Domain: <I>" & strDomain & "</I></div>"
	SchreibDatei "<div>Current Domain Created: <I>" & dtmDomCreateTimestamp & "</I>, Modified: <I>" & dtmDomModifyTimestamp & "</I></div>"
	SchreibDatei "<div>Schema Master (Forest): <I>" & strSchemaMaster & "</I></div>"
	SchreibDatei "<div>Domain Naming Master (Forest): <I>" & strDomainNamingMaster & "</I></div>"
	SchreibDatei "<div>PDC Emulator (Current Domain): <I>" & strPdcEmulator & "</I></div>"
	SchreibDatei "<div>RID Master (Current Domain): <I>" & strRidMaster & "</I></div>"
	SchreibDatei "<div>Infrastructure Master (Current Domain): <I>" & strInfrastructureMaster & "</I></div>"

	SchreibDatei "<BR>"

End Sub


Sub listSites
    ' Funktion: Listet AD-Site-Objekte auf

    objSites.Filter = Array("site")
	intSiteZahl = 0
	For Each objSub In objSites
		intSiteZahl = intSiteZahl + 1
		strDescription = ""
		strLocation = ""
		dtmCreateTimestamp = ""
		dtmModifyTimestamp = ""
	
		strName = objSub.Get("name")
		strDN = objSub.get("distinguishedName")
		dtmCreateTimestamp = objSub.Get("whenCreated")
		dtmModifyTimestamp = objSub.Get("whenChanged")
		' Felder knnten leer sein, daher Fehler abfangen
		On Error Resume Next
			strDescription = objSub.Get("description")
			' ADO returns "description" as an array, so convert it
			if IsArray(strDescription) then strDescription = Join(strDescription, " ")
			strLocation = objSub.Get("location")
		On Error Goto 0
		SchreibDatei "<div class=""site"">" & strSiteSym & "<B>&nbsp;" & strName & "</B>&nbsp;<I>" & strDescription & "</I>"
		SchreibDatei "<div style=""margin-Left:" & 1 * intIndentPx & "px;"">Created: <I>" & dtmCreateTimestamp & "</I>, Modified: <I>" & dtmModifyTimestamp & "</I></div>"
		If strLocation <> "" Then
			SchreibDatei "<div style=""margin-Left:" & 1 * intIndentPx & "px;"">Location: <I>" & strLocation & "</I></div>"
		End If

		' NTDS Site Settings:
		Set objNTDSSS = GetObject("LDAP://CN=NTDS Site Settings, " & strDN)
		strISTG = ""
		strPrefGC = ""
		On Error Resume Next
			strISTG = GetValue(GetValue(objNTDSSS.Get("interSiteTopologyGenerator"), "serverReferenceBL"), "name")
			strPrefGC = GetValue(objNTDSSS.Get("msDS-Preferred-GC-Site"), "name")
		On Error Goto 0
		If strISTG & strPrefGC <> "" Then
			strTxt = "<div style=""margin-Left:" & 1 * intIndentPx & "px;"">"
			strTxt2 = "</div>"
		End If
		If strISTG <> "" Then
			strISTG = "Inter-Site Topology Generator: <I>" & strISTG & "</I>"
		End If
		If strPrefGC <> "" Then
			strPrefGC = "Universal Group Caching from Site: <I>" & strPrefGC & "</I>"
		End If
		SchreibDatei strTxt & strISTG & " " & strPrefGC & strTxt2
		listGPO objSub
		listSubnets strDN
		listServers strDN
		listSiteLinks strDN
		SchreibDatei "</div><!-- site " & strName & " -->" & VbCrLf
	Next
	
End Sub

Function GetValue(strDN, strValue)
	' Funktion: Ermittelt einen AD-Wert zu einem Objekt, das per DN angegeben wird
	If LCase(Left(strDN, 7)) <> "LDAP://" Then
		strDN = "LDAP://" & strDN
	End If
	
	On Error Resume Next
		Set objQuery = GetObject(strDN)
		' Vorsichtshalber den Cache fllen
		objQuery.GetInfoEx Array(strValue),0
		varWert = objQuery.Get(strValue)
	On Error Goto 0
	GetValue = varWert

End Function

Function GetValueArr(arrDN, strValue1)
	' Function: Determines whether strDN contains an array and calls GetValue for each member
	
	If IsArray(arrDN) Then
		Dim arrGV()
		ReDim arrGV(UBound(arrDN))
		For i=0 To UBound(arrDN)
			arrGV(i) = GetValue(arrDN(i), strValue1)
		Next
		GetValueArr = arrGV
	Else
		GetValueArr = GetValue(arrDN, strValue1)
	End If

End Function


Sub listSubnets(strFilterDN)

	strFeld = "name, distinguishedName, location, description, whenCreated, whenChanged"
	strSQL = "SELECT " & strFeld & " FROM 'LDAP://" & strSites & "'" _
		& " WHERE objectClass = 'subnet' and " _
		& " siteObject = '" & strFilterDN & "'"

	intSubnetZahlinSite = 0
	Set objRSSubnet = QueryAD(strSQL)
	if not objRSSubnet.EOF then objRSSubnet.MoveFirst
	
	Do Until objRSSubnet.EOF
		intSubnetZahlinSite = intSubnetZahlinSite + 1
		strLocation = ""
		strDesc = ""
		strSNName = objRSSubnet("name")
		strLocation = objRSSubnet("location")
		strDesc = objRSSubnet("description")
		dtmCreateTimestamp = objRSSubnet("whenCreated")
		dtmModifyTimestamp = objRSSubnet("whenChanged")
		' ADO returns "description" as an array, so convert it
		if IsArray(strDesc) then strDesc = Join(strDesc, " ")
		SchreibDatei "<div style=""margin-Left:" & Round(2 * intIndentPx) & "px;"">" & strSubnetSym & "&nbsp;<B>" & strSNName & "</B>&nbsp;<i>" & strDesc & "</i></div>"
		SchreibDatei "<div style=""margin-Left:" & 3 * intIndentPx & "px;"">Created: <I>" & dtmCreateTimestamp & "</I>, Modified: <I>" & dtmModifyTimestamp & "</I></div>"
		If strLocation <> "" Then 
			SchreibDatei "<div style=""margin-Left:" & Round(3 * intIndentPx) & "px;"">Location: <I>" & strLocation & "</I></div>"
		End If
		objRSSubnet.MoveNext
	Loop
	If intSubnetZahlinSite = 1 Then
		strPlural = ""
	Else
		strPlural = "s"
	End If
	SchreibDatei "<div style=""margin-Left:" & 2 * intIndentPx & "px;"">Found " & intSubnetZahlinSite & " Subnet" & strPlural & " in site.</div>"
	intSubnetZahl = intSubnetZahl + intSubnetZahlinSite

End Sub


Sub listSiteLinks(strFilterDN)

	strFeld = "name, distinguishedName, siteList, cost, replInterval, description, whenCreated, whenChanged"
	strSQL = "SELECT " & strFeld & " FROM 'LDAP://" & strSites & "'" _
		& " WHERE objectClass = 'siteLink' and " _
		& " siteList = '" & strFilterDN & "'"

	intSiteLinkZahlinSite = 0
	Set objRSSiteLink = QueryAD(strSQL)
	if not objRSSiteLink.EOF then objRSSiteLink.MoveFirst
	Do Until objRSSiteLink.EOF
		intSiteLinkZahlinSite = intSiteLinkZahlinSite + 1
		strSLName = objRSSiteLink("name")
		strCost = objRSSiteLink("cost")
		strRepl = objRSSiteLink("replInterval")
		strDesc = objRSSiteLink("description")
		dtmCreateTimestamp = objRSSiteLink("whenCreated")
		dtmModifyTimestamp = objRSSiteLink("whenChanged")
		' ADO returns "description" as an array, so convert it
		if IsArray(strDesc) then strDesc = Join(strDesc, " ")
		SchreibDatei "<div style=""margin-Left:" & Round(2 * intIndentPx) & "px;"">" & strSiteLinkSym & "&nbsp;<B>" & strSLName & "</B>&nbsp;<I>" & strDesc & "</I></DIV>"
		SchreibDatei "<div style=""margin-Left:" & 3 * intIndentPx & "px;"">Created: <I>" & dtmCreateTimestamp & "</I>, Modified: <I>" & dtmModifyTimestamp & "</I></div>"
		SchreibDatei "<div style=""margin-Left:" & Round(3 * intIndentPx) & "px;"">Replication Interval: <I>" & strRepl & " Minutes</I>" _
			& ", Cost: <I>" & strCost & "</I></div>"
		arrSiteList = objRSSiteLink("siteList")
		strWert = ""
		intSiteCount = 0
			If IsArray(arrSiteList) Then
				intSiteCount = UBound(arrSiteList)
				For j = 0 To intSiteCount
					If arrSiteList(j) <> strFilterDN Then
						strSite = GetValue(arrSiteList(j), "name")
						strWert = strWert & "<div style=""margin-Left:" & Round(3.5 * intIndentPx) & "px;""><I> " _
							& strSite & "</I></DIV>" & VbCrLf
					End If
				Next
			Else
				intSiteCount = 1
				If arrSiteList <> strFilterDN Then
					strSite = GetValue(arrSiteList, "name")
					strWert = "<div style=""margin-Left:" & Round(3.5 * intIndentPx) & "px;""><I>" _
						& strSite & "</I></DIV>" & VbCrLf
				End If
			End If
			If strWert <> "" Then
				If intSiteCount = 1 Then
					strPlural = ""
				Else
					strPlural = "s"
				End If
				SchreibDatei "<div style=""margin-Left:" & 3 * intIndentPx & "px;"">" & intSiteCount & " other Site" & strPlural & " linked:</DIV>"
				SchreibDatei strWert
			End If

		objRSSiteLink.MoveNext
	Loop
	If intSiteLinkZahlinSite = 1 Then
		strPlural = ""
	Else
		strPlural = "s"
	End If
	SchreibDatei "<div style=""margin-Left:" & 2 * intIndentPx & "px;"">Found " & intSiteLinkZahlinSite & " Site Link" & strPlural & " in site.</div>" 
	

End Sub

Sub listServers(strDN)

	strServers = "LDAP://CN=Servers, " & strDN
	Set objServers = GetObject(strServers)
	objServers.Filter = Array("server")
	intServerZahlInSite = 0
	
	For Each objSub In objServers
		intServerZahlInSite = intServerZahlInSite + 1
		strName = objSub.Get("name")
		strDNSrv = objSub.get("distinguishedName")
		dtmCreateTimestamp = objSub.Get("whenCreated")
		dtmModifyTimestamp = objSub.Get("whenChanged")
		' Felder knnten leer sein, daher Fehler abfangen
		strDescription = ""
		arrBridgehead = ""
		strBridgehead = ""
		strOS = ""
		On Error Resume Next
			strDescription = objSub.Get("description")
			' ADO returns "description" as an array, so convert it
			if IsArray(strDescription) then strDescription = Join(strDescription, " ")
			arrBridgehead = GetValueArr(objSub.Get("bridgeheadTransportList"), "name")
			strOS = GetValue(objSub.Get("serverReference"), "operatingSystem")
			strOS = strOS & " " & GetValue(objSub.Get("serverReference"), "operatingSystemVersion")
			strOS = strOS & " " & GetValue(objSub.Get("serverReference"), "operatingSystemServicePack")
		On Error Goto 0
		SchreibDatei "<div class=""DC"" style=""margin-Left:" & 2 * intIndentPx & "px;"">" & strDCSym & "<B>&nbsp;" & strName & "</B>&nbsp;<I>" & strDescription & "</I></div>"
		SchreibDatei "<div style=""margin-Left:" & 3 * intIndentPx & "px;"">Created: <I>" & dtmCreateTimestamp & "</I>, Modified: <I>" & dtmModifyTimestamp & "</I></div>"
		If IsArray(arrBridgehead) = False Then
			If arrBridgehead <> "" Then
				strBridgehead = arrBridgehead
			End If
		Else
			strBridgehead = Join(arrBridgehead, ", ")
		End If
		If strBridgehead <> "" Then
			SchreibDatei "<div style=""margin-Left:" & 3 * intIndentPx & "px;"">Pfererred Bridgehead for: <I>" & strBridgehead & "</I></div>"
		End If

		' NTDS Settings data
		strMasterNCs = ""
		arrDSMasterNCs = ""
		arrMasterNCs = ""
		strBehavior = ""
		strQueryPol = ""
		blnEnabled = ""
		strConnName = ""
		strRepDC = ""
		' Server knnte nicht richtig eingerichtet sein (z.B. Testobjekt), daher Fehler abfangen
		On Error Resume Next
			' clear object to initialize
			Set objNTDSSettings = Nothing
			blnNTDS = False

			' Try to fetch NTDS Settings
			Set objNTDSSettings = GetObject("LDAP://CN=NTDS Settings," & strDNSrv)
			' is it a DC? Then it must have NTDS Settings
			If IsObject(objNTDSSettings) And Err.number = 0 Then
				blnNTDS = True
				objNTDSSettings.GetInfoEx Array ("hasMasterNCs", "msDS-hasMasterNCs", "msDS-Behavior-Version", "queryPolicyObject"),0
				arrDSMasterNCs = objNTDSSettings.Get("msDS-hasMasterNCs")
				arrMasterNCs = objNTDSSettings.Get("hasMasterNCs")
				strBehavior = objNTDSSettings.Get("msDS-Behavior-Version")
				intOptions = ""
				intOptions = objNTDSSettings.Get("options")
				strQueryPol = GetValue(objNTDSSettings.Get("queryPolicyObject"), "name")
				objNTDSSettings.Filter = Array("nTDSConnection")
				intCount = 0
				' Find Replication Connections
				For Each objConn In objNTDSSettings
					objConn.GetInfoEx Array("fromServer"),0
					ReDim preserve arrConn(intCount)
					strConnName = objConn.Get("name")
					blnConnEnabled = objConn.Get("enabledConnection")
					If blnConnEnabled Then
						strEnabled = " (enabled)"
					Else
						strEnabled = " (disabled)"
					End If
					strRepDC = Replace(objConn.Get("fromServer"), "CN=NTDS Settings,", "")
					strRepDC = GetValue(strRepDC, "name")
					arrConn(intCount) = "<div style=""margin-Left:" & Round(3.5 * intIndentPx) & "px;"">Partner: <I>" & strRepDC _
						& "</I>, Connection: <I>" & strConnName & strEnabled & "</I></div>"
					intCount = intCount + 1
				Next
				If intOptions And NTDSDSA_OPT_IS_GC Then
					SchreibDatei "<div style=""margin-Left:" & 3 * intIndentPx & "px;"">Server is <I>Global Catalog Server</I></div>"
				End If
			Else
				SchreibDatei "<div style=""margin-Left:" & 3 * intIndentPx & "px;"">Server does not seem to be a Domain Controller (no NTDS Settings found).</I></div>"
			End If
			Set objNTDSSettings = Nothing
		On Error Goto 0

		' NCs held by this DC
		If IsArray(arrDSMasterNCs) = False Then
			If arrDSMasterNCs = "" Then
				' msDS-hasMasterNSs not in use, use hasMasterNCs
				If IsArray(arrMasterNCs) = False Then
					If arrMasterNCs <> "" Then
						strMasterNCs = "<div style=""margin-Left:" & Round(3.5 * intIndentPx) & "px;"">" & "&nbsp;<I>" & arrMasterNCs & "</I></div>"
					End If
				Else
					For j = 0 To UBound(arrMasterNCs)
						If arrMasterNCs(j) <> "" Then
							strMasterNCs = strMasterNCs & "<div style=""margin-Left:" & Round(3.5 * intIndentPx) & "px;"">" & "&nbsp;<I>" & arrMasterNCs(j) & "</I></div>"
						End If
					Next
				End If
			Else
				strMasterNCs = "<div style=""margin-Left:" & Round(3.5 * intIndentPx) & "px;"">" & "&nbsp;<I>" & arrDSMasterNCs & "</I></div>"
			End If
		Else
			For j = 0 To UBound(arrDSMasterNCs)
				If arrDSMasterNCs(j) <> "" Then
					strMasterNCs = strMasterNCs & "<div style=""margin-Left:" & Round(3.5 * intIndentPx) & "px;"">" & "&nbsp;<I>" & arrDSMasterNCs(j) & "</I></div>"
				End If
			Next
    	End If
		If strMasterNCs <> "" Then
			SchreibDatei "<div class=""MasterNCs"" style=""margin-Left:" & 3 * intIndentPx & "px;"">" & "Naming Contexts:</div>"
			SchreibDatei strMasterNCs
		End If

		If strQueryPol <> "" Then
			SchreibDatei "<div style=""margin-Left:" & 3 * intIndentPx & "px;"">Query Policy: <I>" & strQueryPol & "</I></div>"
		End If

		If strOS <> "" Then
			SchreibDatei "<div style=""margin-Left:" & 3 * intIndentPx & "px;"">Operating System: <I>" & strOS & "</I></div>"
		End If

		' AD functional version
		If strBehavior <> "" Then
			Select Case strBehavior
				Case "0" strBehavior = "Windows 2000"
				Case "1" strBehavior = "Windows Server 2003"
				Case "2" strBehavior = "Windows Server 2003"
				Case "3" strBehavior = "Windows Server 2008"
				Case "4" strBehavior = "Windows Server 2008 R2"
				Case "5" strBehavior = "Windows Server 2012"
				Case "6" strBehavior = "Windows Server 2012 R2"
				Case "7" strBehavior = "Windows Server 2016"
				Case Else strBehavior = "AD Mode " & strBehavior
			End Select
			SchreibDatei "<div style=""margin-Left:" & 3 * intIndentPx & "px;"">Functional Mode: <I>" & strBehavior & "</I></div>"
		End If

		' List Replication Connections
		If IsArray(arrConn) Then
			If intCount = 1 Then
				strPlural = ""
			Else
				strPlural = "s"
			End If
			SchreibDatei "<div style=""margin-Left:" & 3 * intIndentPx & "px;"">" & intCount & " Replication Connection" & strPlural & ":</div>"
			For j = 0 To UBound(arrConn)
				SchreibDatei arrConn(j)
			Next
		Else
			If arrConn <> "" Then
				SchreibDatei "<div style=""margin-Left:" & 3 * intIndentPx & "px;"">Replication Connection:</div>"
				SchreibDatei arrConn
			End If
		End If
	Next
	If intServerZahlInSite = 1 Then
		strPlural = ""
	Else
		strPlural = "s"
	End If
	SchreibDatei "<div style=""margin-Left:" & 2 * intIndentPx & "px;"">Found " & intServerZahlInSite & " server" & strPlural & " in site.</div>"
	intServerZahl = intServerZahl + intServerZahlInSite

End Sub


Function QueryAD(strSQL)
	Dim objConn
    Dim objCommand
    Dim objArgs

	Set QueryAD = Nothing
	
    On Error Resume Next
    'Create ADO connection object for Active Directory
    Set objConn = CreateObject("ADODB.Connection")
    objConn.Provider = "ADsDSOObject"
    objConn.Open "Active Directory Provider"
    if checkit("Error on ADO Access to Active Directory!") then Exit Function

    'Create ADO command object for the connection.
    Set objCommand = CreateObject("ADODB.Command")
    objCommand.ActiveConnection = objConn
    if checkit("Error on Definition of ADO Command!") then Exit Function

    'Assemble the commandtext.
    objCommand.CommandText = strSQL
    if checkit("Error on Passing the Command!") then Exit Function

    'Execute the query.
	objCommand.Properties("Page Size") = 1000
	Set objRS = objCommand.Execute
    if checkit("Error Executing Active Directory Search!") then Exit Function

	Set QueryAD = objRS
    if checkit("Error on Copying the Recordset!") then Exit Function
	
    On Error Goto 0

End Function


Sub AusgabeDatei()
    ' Funktion: Legt die Ausgabedatei an und hlt sie offen
    ' Eingabeparameter:
    ' Kommentar:
    strScriptPfad = WScript.ScriptFullName
    strDateiPfad = Left(strScriptPfad, InStrRev(strScriptPfad, "\")) & "Reports"

    Set objFSO = CreateObject("Scripting.FileSystemObject")
    With objFSO
    	dateiName = "AD-Sites " & CStr(Now) & ".htm"
    	dateiName = Replace(dateiName, ":", "-")
		dateiName = Replace(dateiName, "/", "-")
    	pfadName = .BuildPath(strDateiPfad, dateiName)
		If Not .FolderExists(strDateiPfad) Then
			.CreateFolder strDateiPfad
		End If
    	Set objDatei = .CreateTextFile(pfadName)
    End With

    SchreibDatei "<HTML>"
    SchreibDatei "<HEAD>"
    SchreibDatei "  <LINK REL=""stylesheet"" type=""text/css"" href=""../Dateien/styles.css"">"
    SchreibDatei "</HEAD>"
    SchreibDatei "<BODY>"
	SchreibDatei "<H1>AD Site Configuration: " & strSites & "</H1>"
	SchreibDatei "<BR>"

	say strTitel & " launched at " & Now
	say "Report File: " & pfadName

End Sub

Sub listGPO(objStart)
    ' Funktion: listet die GPO-Links eines Containers
    ' Eingabeparameter: objStart: untersuchter Container
    '                   strSymbol: Anzuzeigendes Symbol
    ' Kommentar:
	
    arrGPOLinks = ""
	On Error Resume Next
    	arrGPOLinks = objStart.Get("gPLink")
    On Error Goto 0
    if arrGPOLinks = "" then Exit Sub
    arrGPOLinks = Split(arrGPOLinks, "][")
	' there might be non-empty links that do not link to GPOs
	' so we must count separately
	intCountGPOLink = 0
    For i=0 To UBound(arrGPOLinks)
      If InStr(arrGPOLinks(i), "{") > 0 Then
        intCountGPOLink = intCountGPOLink + 1
		arrGPOLinks(i) = Replace(Replace(arrGPOLinks(i), "[", ""), "]", "")
        links  = InStr(arrGPOLinks(i), "{")
        strGPOGUIDName = Mid(arrGPOLinks(i), links)
        rechts = InStrRev(strGPOGUIDName, "}")
        strGPOGUIDName = Left(strGPOGUIDName, rechts)
        strGPODisplayName = GPOName(strGPOGUIDName)
        strGPOName = strGPOSym & "&nbsp;<B>" & strGPODisplayName & "</B>&nbsp;<I>(" & strGPOGUIDName & ")</I>"
        SchreibDatei "<div class=""GPO"" style=""margin-left:" & intIndentPx & "px;"">" & strGPOName & "</div>"
      End If
    Next
	If intCountGPOLink = 1 Then
		strPlural = ""
	Else
		strPlural = "s"
	End If
	If intCountGPOLink <> 0 Then
		SchreibDatei "<div style=""margin-Left:" & intIndentPx & "px;"">Found " & intCountGPOLink & " GPO link" & strPlural & " in site.</div>"
	End If

End Sub

Function GPOName(strGPOGUID)
    ' Funktion: Ermittelt den "displayName" eines GPO bei gegebenem GUID-Namen
    ' Eingabeparameter: strGPOGUID: Angabe des GUID-Namens
    ' Kommentar:
    Set objGPOCont = GetObject("LDAP://CN=Policies,CN=System," & strRootDom)
    objGPOCont.filter = Array("groupPolicyContainer")
    For Each objGPO In objGPOCont
        If InStr(objGPO.name, strGPOGUID) > 0 Then GPOName = objGPO.get("displayName")
    Next
    Set objGPOCint = Nothing

End Function

Sub listSymbols
	SchreibDatei "<BR clear=""all""><HR>"
	SchreibDatei "<H2>Symbols</H2>"
	SchreibDatei "<div>" & strSiteSym & "&nbsp;Site</div>"
	SchreibDatei "<div>" & strGPOSym & "&nbsp;Group Policy Object</div>"
	SchreibDatei "<div>" & strSubnetSym & "&nbsp;Subnet</div>"
	SchreibDatei "<div>" & strDCSym & "&nbsp;Domain Controller</div>"
	SchreibDatei "<div>" & strSiteLinkSym & "&nbsp;Site Link</div>"

End Sub


Sub SchreibDatei(strText)
    ' Funktion: schreibt angegebenen Text in die Logdatei
    ' Eingabeparameter: strText: Zu schreibender String
    ' Kommentar:

    objDatei.WriteLine strText

End Sub


Sub dateiZu()
    ' Funktion:
    ' Eingabeparameter:
    ' Kommentar:
	strSQL = "SELECT * FROM 'LDAP://" & strSites & "'" _
	& " WHERE objectClass = 'siteLink'"
	Set objRSStat = QueryAD(strSQL)
	if not objRSStat.EOF then objRSStat.MoveFirst
	intSiteLinkZahl = 0
	Do Until objRSStat.EOF
		intSiteLinkZahl = intSiteLinkZahl + 1
		objRSStat.MoveNext
	Loop

    SchreibDatei "<BR><HR><H2>Statistics</H2>" & VbCrLf
	SchreibDatei "<div>Number of Sites: " & intSiteZahl & "</div>"
	SchreibDatei "<div>Number of Site Links: " & intSiteLinkZahl & "</div>"
	SchreibDatei "<div>Number of Subnets: " & intSubnetZahl & "</div>"
	SchreibDatei "<div>Number of Servers: " & intServerZahl & "</div>"

	SchreibDatei "<BR><HR><p>Documented on " & Now _
		& " using <a href=""http://www.kaczenski.de"">" & strTitel & "</a> by Nils Kaczenski</P>"
	SchreibDatei "</BODY>"
    SchreibDatei "</HTML>"

    objDatei.Close
	If blnShowReport Then
		Set objIE = CreateObject("InternetExplorer.Application")
		objIE.Navigate pfadName
		objIE.Visible = vbTrue
	End If


End Sub


' Fehlerprfung
Function checkit(strNachricht)
	checkit = False
	If Err.number <>0 Then
		strNachricht = strNachricht & " [" & Err.description & " (" & Err.number & ")]"
            If ADSIError Then
               Set objError = CreateObject("ADsError")
               strKlartext = objError.GetErrorMessage(Err.number)
               strNachricht = strNachricht & VbCrLf & "ADSI Error: " & strKlartext & " (0x" & Hex(Err.number) & ")"
            End If
            sayerror strNachricht
		Err.clear
		checkit=True
	End If
End Function

' Universelle Ausgabefunktion
Sub say(s)
	saynb s '& VbCrLf
End Sub

' Ausgabe ohne Zeilenumbruch
Sub saynb(s)
	WScript.echo s
End Sub

' Ausgabe als Fehlermeldung
Sub sayerror(s)
	say "Error (" & Date & ", " & Time & "): " & s
End Sub

' Ausgabe als Warnung
Sub sayalert(s)
	say "Warnung (" & Date & ", " & Time & "): " & s
End Sub


' Debug-Ausgabe
Sub saydebug(s)
	If debugmode Then
		say s
	End If
End Sub

